@@ -31,7 +31,7 @@ from qtpy.QtCore import Qt, QTimer
 from qtpy.QtWidgets import QFileDialog, QMessageBox, QPushButton, QApplication
 from opensesame_ide import FolderBrowserDockWidget, MenuBar
 from libqtopensesame.misc.translate import translation_context
-from pyqode.core import widgets
+from pyqode.core import widgets, panels
 from pyqode.core.api.utils import TextHelper
 from pyqode_extras.widgets import FallbackCodeEdit
 _ = translation_context(u'OpenSesameIDE', category=u'extension')
@@ -80,11 +80,14 @@ class OpenSesameIDE(BaseExtension):
         self._register_mimetypes()
         self._patch_behavior()
         self._scetw = widgets.SplittableCodeEditTabWidget(self.main_window)
-        self._scetw.main_tab_widget.cornerWidget().hide()
         self._scetw.fallback_editor = FallbackCodeEdit
         self._scetw.tab_name = u'OpenSesameIDE'
-        self._scetw.main_tab_widget.tab_closed.connect(self._on_editor_close)
         self._scetw.tab_bar_visible = cfg.opensesame_ide_show_tab_bar
+        self._patch_splitter(self._scetw)
+        # self._scetw.main_tab_widget.tab_closed.connect(self._on_editor_close)
+        # self._scetw.editor_created.connect(self._on_editor_created)
+        self._scetw.default_extension = \
+            lambda: cfg.opensesame_ide_default_extension
         self._add_ide_tab()
         self._dock_widgets = {}
         self._set_ignore_patterns()
@@ -149,6 +152,18 @@ class OpenSesameIDE(BaseExtension):
 
         cfg.opensesame_ide_show_tab_bar = show_tab_bar
         self._scetw.tab_bar_visible = show_tab_bar
+        
+    def event_register_editor(self, editor, mime_type=u'text/plain'):
+        """The IDE mode has a special panel that allows the extension to be
+        changed for empty unsaved buffers.
+        """
+        try:
+            editor.panels.get('ChangeExtensionPanel')
+        except KeyError:
+            editor.panels.append(
+                panels.ChangeExtensionPanel(),
+                panels.ChangeExtensionPanel.Position.TOP
+            )
 
     def event_ide_new_file(self, source=None, ext=None):
 
@@ -358,10 +373,6 @@ class OpenSesameIDE(BaseExtension):
         if not ext:
             ext = cfg.opensesame_ide_default_extension
         editor = self._current_splitter().create_new_document(extension=ext)
-        self.extension_manager.fire(
-            u'register_editor',
-            editor=editor
-        )
 
     def save_file(self):
 
@@ -389,9 +400,11 @@ class OpenSesameIDE(BaseExtension):
 
     def save_file_as(self):
 
+        current_splitter = self._current_splitter()
         editor = self._current_editor()
+        current_splitter.main_tab_widget.unpin(editor)
         from_path = editor.file.path if editor else ''
-        self._current_splitter().save_current_as()
+        current_splitter.save_current_as()
         editor = self._current_editor()
         if editor is None:
             return
@@ -750,9 +763,7 @@ class OpenSesameIDE(BaseExtension):
         # If there are no child splitters, we use the regular split() method
         if not splitter.child_splitters:
             subsplitter = splitter.split(editor, direction)
-            subsplitter.main_tab_widget.tab_closed.connect(
-                self._on_editor_close
-            )
+            self._patch_splitter(subsplitter)
             self.extension_manager.fire(
                 u'register_editor',
                 editor=self._current_editor()
@@ -765,16 +776,14 @@ class OpenSesameIDE(BaseExtension):
         # pyqode.
         self.main_window.setUpdatesEnabled(False)
         subsplitter = splitter.split(editor, splitter.orientation(), index=1)
-        subsplitter.main_tab_widget.tab_closed.connect(self._on_editor_close)
+        self._patch_splitter(subsplitter)
         editor = self._current_editor()
         self.extension_manager.fire(
             u'register_editor',
             editor=editor
         )
         subsubsplitter = subsplitter.split(editor, direction, index=1)
-        subsubsplitter.main_tab_widget.tab_closed.connect(
-            self._on_editor_close
-        )
+        self._patch_splitter(subsubsplitter)
         editor = self._current_editor()
         self.extension_manager.fire(
             u'register_editor',
@@ -916,6 +925,19 @@ class OpenSesameIDE(BaseExtension):
             return fnc(widget, *args, **kwargs)
 
         return inner
+    
+    def _patch_splitter(self, splitter):
+        
+        tw = splitter.main_tab_widget
+        tw.tab_closed.connect(self._on_editor_close)
+        tw.split_requested.disconnect(splitter.split)
+        tw.split_requested.connect(self._on_split_requested)
+        splitter.editor_created.connect(self._on_editor_created)
+        
+    def _on_split_requested(self, widget, orientation, index=None):
+        
+        widget.setFocus()
+        self._split(orientation)
 
     def _on_tabwidget_close(self, index):
 
@@ -1059,6 +1081,23 @@ class OpenSesameIDE(BaseExtension):
     def provide_ide_current_path(self):
 
         return self._current_path()
+        
+    def _on_extension_changed(self, ext):
+        
+        print(ext)
+        cfg.opensesame_ide_default_extension = ext
+        
+    def _on_editor_created(self, editor):
+        self.extension_manager.fire(
+            u'register_editor',
+            editor=editor
+        )
+        try:
+            p = editor.panels.get('ChangeExtensionPanel')
+        except KeyError:
+            pass
+        else:
+            p.extension_changed.connect(self._on_extension_changed)
 
     @with_editor
     def _current_tabwidget(self, editor):
