@@ -86,6 +86,10 @@ class MenuBar(QMenuBar):
         self._menu_file.addAction(self._action_close_all_folders)
         self._menu_file.addSeparator()
         self._menu_file.addAction(self._action_quit)
+        # Edit menu
+        self._menu_edit = self.addMenu(_(u'&Edit'))
+        self._menu_edit.aboutToShow.connect(self._show_edit_menu)
+        self._menu_edit.aboutToHide.connect(self._menu_edit.clear)
         # Tools menu
         self._action_preferences = self._action(
             _(u'&Preferences'),
@@ -111,192 +115,36 @@ class MenuBar(QMenuBar):
             'GitGUI',
             menu=self._menu_tools
         )
-        # View menu
-        self._action_close_tab = self._action(
-            _(u'&Close tab'),
-            u'window-close',
-            cfg.opensesame_ide_shortcut_close_tab,
-            ide.close_tab
-        )
-        self._action_close_other_tabs = self._action(
-            _(u'Close &other tabs'),
-            u'window-close',
-            cfg.opensesame_ide_shortcut_close_other_tabs,
-            ide.close_other_tabs
-        )
-        self._action_close_all_tabs = self._action(
-            _(u'Close &all tabs'),
-            u'window-close',
-            cfg.opensesame_ide_shortcut_close_all_tabs,
-            ide.close_all_tabs
-        )
-        self._action_split_vertical = self._action(
-            _(u'Split &vertical'),
-            u'go-down',
-            cfg.opensesame_ide_shortcut_split_vertical,
-            ide.split_vertical
-        )
-        self._action_split_horizontal = self._action(
-            _(u'Split &horizontal'),
-            u'go-next',
-            cfg.opensesame_ide_shortcut_split_horizontal,
-            ide.split_horizontal
-        )
-        self._action_switch_splitter_previous = self._action(
-            _(u'Switch to previous panel'),
-            u'go-previous',
-            cfg.opensesame_ide_shortcut_switch_previous_panel,
-            ide.switch_splitter_previous
-        )
-        self._action_switch_splitter_next = self._action(
-            _(u'Switch to next panel'),
-            u'go-next',
-            cfg.opensesame_ide_shortcut_switch_next_panel,
-            ide.switch_splitter_next
-        )
-        self._action_toggle_fullscreen = self._action(
-            _(u'Toggle fullscreen'),
-            u'view-fullscreen',
-            cfg.opensesame_ide_shortcut_toggle_fullscreen,
-            ide._toggle_fullscreen,
-            checkable=True
-        )
-        self._action_toggle_folder_browsers = self._action(
-            _(u'Toggle &folder browsers'),
-            u'os-overview',
-            cfg.opensesame_ide_shortcut_toggle_folder_browsers,
-            ide.toggle_folder_browsers,
-            checkable=True
-        )
-        self._action_locate_file_in_folder = self._action(
-            _(u'&Locate active file'),
-            u'folder',
-            cfg.opensesame_ide_shortcut_locate_active_file,
-            ide.locate_file_in_folder
-        )
-        self._menu_view = self.addMenu(_('&View'))
-        self._menu_view.addAction(self._action_close_tab)
-        self._menu_view.addAction(self._action_close_other_tabs)
-        self._menu_view.addAction(self._action_close_all_tabs)
-        self._menu_view.addSeparator()
-        self._menu_view.addAction(self._action_split_vertical)
-        self._menu_view.addAction(self._action_split_horizontal)
-        self._menu_view.addAction(self._action_switch_splitter_previous)
-        self._menu_view.addAction(self._action_switch_splitter_next)
-        self._menu_view.addSeparator()
-        self._menu_view.addAction(self._action_toggle_fullscreen)
-        self._menu_view.addSeparator()
-        self._menu_view.addAction(self._action_toggle_folder_browsers)
-        self._menu_view.addAction(self._action_locate_file_in_folder)
-        self._action_toggle_console = self._add_extension_action(
-            'JupyterConsole',
-            menu=self._menu_view,
-            separate=True
-        )
-        self._action_toggle_workspace = self._add_extension_action(
-            'WorkspaceExplorer',
-            menu=self._menu_view,
-        )
         self._action_symbol_selector = self._add_extension_action(
             'SymbolSelector',
-            menu=self._menu_view,
+            menu=self._menu_tools,
             separate=True
         )
         self._action_find_in_files = self._add_extension_action(
             'FindInFiles',
-            menu=self._menu_view,
-            separate=True
+            menu=self._menu_tools,
+            separate=False
         )
         self._action_command_palette = self._add_extension_action(
             'CommandPalette',
-            menu=self._menu_view,
-            separate=True
+            menu=self._menu_tools,
+            separate=False
         )
-        # Run menu
-        self._menu_run = self.addMenu(_('&Run'))
-        self._action_run_current_file = self._action(
-            _(u'&Run project or file'),
-            u'os-run',
-            cfg.opensesame_ide_shortcut_run_file,
-            ide.run_current_file,
+        self._action_spellcheck = self._add_extension_action(
+            'SpellCheck',
+            menu=self._menu_tools,
+            separate=False
         )
-        self._action_run_current_selection = self._action(
-            _(u'Run &selection, cell, or current line'),
-            u'os-run-quick',
-            cfg.opensesame_ide_shortcut_run_selection,
-            ide.run_current_selection,
+        self._action_word_count = self._add_extension_action(
+            'WordCount',
+            menu=self._menu_tools,
+            separate=False
         )
-        self._action_run_from_current_position = self._action(
-            _(u'Run &from current position'),
-            u'os-run-quick',
-            cfg.opensesame_ide_shortcut_run_from_current_position,
-            ide.run_from_current_position,
+        # Editor menu (submenu of view)
+        self._menu_editor = QMenu(_('&Editor'), self)
+        self._menu_editor.setIcon(
+            self._ide.theme.qicon(u'accessories-text-editor')
         )
-        self._action_run_up_to_current_position = self._action(
-            _(u'Run &up to current position'),
-            u'os-run-quick',
-            cfg.opensesame_ide_shortcut_run_up_to_current_position,
-            ide.run_up_to_current_position,
-        )
-        self._action_capture_output = self._cfg_action(
-            self._menu_run,
-            _(u'&Capture output'),
-            u'image_annotations_capture_output'
-        )
-        self._action_run_debug = self._action(
-            _(u'Run file in &debugger'),
-            u'os-run',
-            cfg.opensesame_ide_shortcut_run_debug,
-            ide.run_debug,
-        )
-        self._action_toggle_breakpoint = self._action(
-            _(u'&Toggle breakpoint'),
-            u'list-add',
-            cfg.opensesame_ide_shortcut_toggle_breakpoint,
-            ide.toggle_breakpoint,
-        )
-        self._action_clear_breakpoints = self._action(
-            _(u'Clear &breakpoints'),
-            u'list-remove',
-            cfg.opensesame_ide_shortcut_clear_breakpoints,
-            ide.clear_breakpoints,
-        )
-        self._action_run_interrupt = self._action(
-            _(u'&Interrupt kernel'),
-            u'os-kill',
-            cfg.opensesame_ide_shortcut_run_interrupt,
-            ide.run_interrupt,
-        )
-        self._action_run_restart = self._action(
-            _(u'Restart &kernel'),
-            u'view-refresh',
-            None,
-            ide.run_restart,
-        )
-        self._action_change_working_directory = self._action(
-            _(u'Change &working directory to active file'),
-            u'folder-open',
-            cfg.opensesame_ide_shortcut_change_working_directory,
-            ide.change_working_directory,
-        )
-        # Run menu
-        self._menu_run.addAction(self._action_run_current_file)
-        self._menu_run.addAction(self._action_run_current_selection)
-        self._menu_run.addAction(self._action_run_from_current_position)
-        self._menu_run.addAction(self._action_run_up_to_current_position)
-        self._menu_run.addSeparator()
-        self._menu_run.addAction(self._action_capture_output)
-        self._menu_run.addSeparator()
-        self._menu_run.addAction(self._action_run_interrupt)
-        self._menu_run.addAction(self._action_run_restart)
-        self._menu_run.addSeparator()
-        self._menu_run.addAction(self._action_change_working_directory)
-        self._menu_run.addSeparator()
-        self._menu_run.addAction(self._action_run_debug)
-        self._menu_run.addAction(self._action_toggle_breakpoint)
-        self._menu_run.addAction(self._action_clear_breakpoints)
-        # Editor menu
-        self._menu_editor = self.addMenu(_('&Editor'))
         self._action_toggle_line_wrap = self._cfg_action(
             self._menu_editor,
             _(u'Wrap lines'),
@@ -348,16 +196,202 @@ class MenuBar(QMenuBar):
         )
         self._menu_editor.addAction(self._action_toggle_tab_bar)
         self._menu_editor.addAction(self._action_select_indentation_mode)
-        self._action_word_count = self._add_extension_action(
-            'SpellCheck',
-            menu=self._menu_editor,
+        # Tabs menu (submenu of view)
+        self._menu_tabs = QMenu(_('&Tabs'), self)
+        self._menu_tabs.setIcon(
+            self._ide.theme.qicon(u'accessories-text-editor')
+        )
+        self._action_close_tab = self._action(
+            _(u'&Close tab'),
+            u'window-close',
+            cfg.opensesame_ide_shortcut_close_tab,
+            ide.close_tab
+        )
+        self._action_close_other_tabs = self._action(
+            _(u'Close &other tabs'),
+            u'window-close',
+            cfg.opensesame_ide_shortcut_close_other_tabs,
+            ide.close_other_tabs
+        )
+        self._action_close_all_tabs = self._action(
+            _(u'Close &all tabs'),
+            u'window-close',
+            cfg.opensesame_ide_shortcut_close_all_tabs,
+            ide.close_all_tabs
+        )
+        self._action_split_vertical = self._action(
+            _(u'Split &vertical'),
+            u'go-down',
+            cfg.opensesame_ide_shortcut_split_vertical,
+            ide.split_vertical
+        )
+        self._action_split_horizontal = self._action(
+            _(u'Split &horizontal'),
+            u'go-next',
+            cfg.opensesame_ide_shortcut_split_horizontal,
+            ide.split_horizontal
+        )
+        self._action_switch_splitter_previous = self._action(
+            _(u'Switch to previous panel'),
+            u'go-previous',
+            cfg.opensesame_ide_shortcut_switch_previous_panel,
+            ide.switch_splitter_previous
+        )
+        self._action_switch_splitter_next = self._action(
+            _(u'Switch to next panel'),
+            u'go-next',
+            cfg.opensesame_ide_shortcut_switch_next_panel,
+            ide.switch_splitter_next
+        )
+        self._menu_tabs.addAction(self._action_close_tab)
+        self._menu_tabs.addAction(self._action_close_other_tabs)
+        self._menu_tabs.addAction(self._action_close_all_tabs)
+        self._menu_tabs.addSeparator()
+        self._menu_tabs.addAction(self._action_split_vertical)
+        self._menu_tabs.addAction(self._action_split_horizontal)
+        self._menu_tabs.addAction(self._action_switch_splitter_previous)
+        self._menu_tabs.addAction(self._action_switch_splitter_next)
+        # View menu
+        self._action_toggle_fullscreen = self._action(
+            _(u'Toggle fullscreen'),
+            u'view-fullscreen',
+            cfg.opensesame_ide_shortcut_toggle_fullscreen,
+            ide._toggle_fullscreen,
+            checkable=True
+        )
+        self._action_toggle_folder_browsers = self._action(
+            _(u'Toggle &folder browsers'),
+            u'os-overview',
+            cfg.opensesame_ide_shortcut_toggle_folder_browsers,
+            ide.toggle_folder_browsers,
+            checkable=True
+        )
+        self._action_locate_file_in_folder = self._action(
+            _(u'&Locate active file'),
+            u'folder',
+            cfg.opensesame_ide_shortcut_locate_active_file,
+            ide.locate_file_in_folder
+        )
+        self._menu_view = self.addMenu(_('&View'))
+        self._menu_view.addMenu(self._menu_editor)
+        self._menu_view.addMenu(self._menu_tabs)
+        self._menu_view.addSeparator()
+        self._menu_view.addAction(self._action_toggle_fullscreen)
+        self._menu_view.addSeparator()
+        self._menu_view.addAction(self._action_toggle_folder_browsers)
+        self._menu_view.addAction(self._action_locate_file_in_folder)
+        self._action_toggle_console = self._add_extension_action(
+            'JupyterConsole',
+            menu=self._menu_view,
             separate=True
         )
-        self._action_word_count = self._add_extension_action(
-            'WordCount',
-            menu=self._menu_editor,
-            separate=True
+        self._action_toggle_workspace = self._add_extension_action(
+            'WorkspaceExplorer',
+            menu=self._menu_view,
         )
+        # Run menu
+        self._menu_run = self.addMenu(_('&Run'))
+        self._action_run_current_file = self._action(
+            _(u'&Run project or file'),
+            u'os-run',
+            cfg.opensesame_ide_shortcut_run_file,
+            ide.run_current_file,
+        )
+        self._action_run_current_selection = self._action(
+            _(u'Run &selection, cell, or current line'),
+            u'os-run-quick',
+            cfg.opensesame_ide_shortcut_run_selection,
+            ide.run_current_selection,
+        )
+        self._action_run_from_current_position = self._action(
+            _(u'Run &from current position'),
+            u'os-run-quick',
+            cfg.opensesame_ide_shortcut_run_from_current_position,
+            ide.run_from_current_position,
+        )
+        self._action_run_up_to_current_position = self._action(
+            _(u'Run &up to current position'),
+            u'os-run-quick',
+            cfg.opensesame_ide_shortcut_run_up_to_current_position,
+            ide.run_up_to_current_position,
+        )
+        self._action_capture_output = self._cfg_action(
+            self._menu_run,
+            _(u'&Capture output'),
+            u'image_annotations_capture_output'
+        )
+        self._action_clear_output = self._action(
+            _(u'C&lear output'),
+            u'edit-clear',
+            None,
+            lambda: self._ide.extension_manager.fire(
+                'image_annotations_clear_output'
+            )
+        )
+        self._action_run_debug = self._action(
+            _(u'Run file in &debugger'),
+            u'os-run',
+            cfg.opensesame_ide_shortcut_run_debug,
+            ide.run_debug,
+        )
+        self._action_toggle_breakpoint = self._action(
+            _(u'&Toggle breakpoint'),
+            u'list-add',
+            cfg.opensesame_ide_shortcut_toggle_breakpoint,
+            ide.toggle_breakpoint,
+        )
+        self._action_clear_breakpoints = self._action(
+            _(u'Clear &breakpoints'),
+            u'list-remove',
+            cfg.opensesame_ide_shortcut_clear_breakpoints,
+            ide.clear_breakpoints,
+        )
+        self._action_run_interrupt = self._action(
+            _(u'&Interrupt kernel'),
+            u'os-kill',
+            cfg.opensesame_ide_shortcut_run_interrupt,
+            ide.run_interrupt,
+        )
+        self._action_run_restart = self._action(
+            _(u'Restart &kernel'),
+            u'view-refresh',
+            None,
+            ide.run_restart,
+        )
+        self._action_change_working_directory = self._action(
+            _(u'Change &working directory to active file'),
+            u'folder-open',
+            cfg.opensesame_ide_shortcut_change_working_directory,
+            ide.change_working_directory,
+        )
+        # Run menu
+        self._menu_run.addAction(self._action_run_current_file)
+        self._menu_run.addAction(self._action_run_current_selection)
+        self._menu_run.addAction(self._action_run_from_current_position)
+        self._menu_run.addAction(self._action_run_up_to_current_position)
+        self._menu_run.addSeparator()
+        self._menu_run.addAction(self._action_capture_output)
+        self._menu_run.addAction(self._action_clear_output)
+        self._menu_run.addSeparator()
+        self._menu_run.addAction(self._action_run_interrupt)
+        self._menu_run.addAction(self._action_run_restart)
+        self._menu_run.addSeparator()
+        self._menu_run.addAction(self._action_change_working_directory)
+        self._menu_run.addSeparator()
+        self._menu_run.addAction(self._action_run_debug)
+        self._menu_run.addAction(self._action_toggle_breakpoint)
+        self._menu_run.addAction(self._action_clear_breakpoints)
+        
+    def _show_edit_menu(self):
+        
+        editor = self._ide._current_editor()
+        self._menu_edit.clear()
+        if editor is None:
+            action = self._menu_edit.addAction(_(u'No active editor'))
+            action.setEnabled(False)
+            return
+        for action in editor.get_context_menu().actions():
+            self._menu_edit.addAction(action)
 
     def _add_extension_action(self, ext, menu, separate=False):
 
